function [P,t,A]=parties(S,cho)

% [P,t,A]=parties(S,cho)
%
% *********************************************************
%  Parties d'un ensemble
%  * Variables d'entres :
%     *  S : cellule (ne marche que si S est une cellule du type
%        num2cell(X) o X tableau numrique)
%  * Variables d'entres optionnelles
%     * cho : entier dans {0,1,2} (0 par dfaut)
%        * 0 : calcul par rcurrence
%        * 1 : avec produit_cartesien
%        * 2 : avec nchoosek
%  * Variables de sorties
%        * P : cellules des parties
%  * Variables de sorties optionnelles
%       * t : tableau pour produit cartsien (sinon, vide)
%       * A : cellule pour utilisation par stocke_tableau_tex
%
% Exemple
%   a)
%      demoparties(3)
%   b)
%      n=4;
%      [P,t,A]=parties(num2cell(1:n),1);
%      stocke_tableau_tex(A,'l',[],[],[],[],[],[],4,2,['tabparties',int2str(n)],{['tabparties',int2str(n)],['Parties d''un ensemble  ' ,int2str(n),' lments'],1},{'nombres en binaires','parties'});
%
%
% ************ Fonctions auxiliaires utilises ************
%
%    produit_cartesien, affiche_ensemble03
%
% *********************************************************
%
% 2017 by          Jrme BASTIEN
%                  Universit Claude Bernard Lyon I, Polytech, Laboratoire LIBM, Villeurbanne
%                  E-Mail : jerome.bastien@univ-lyon1.fr

nout=nargout;
if isempty(S)
    P={[]};
    if nout>=2
        if cho==1
            t=0;
        else
            t=[];
        end
    end
else
    if nargin<=1||isempty(cho)
        cho=0;
    end
    switch cho
        case 0
            Q=parties(S(1:end-1));
            lQ=length(Q);
            u=cell(1,lQ);
            an=S{end};
            % vectorizer ?
            for i=1:lQ
                u{i}=[Q{i} an];
            end
            P=[Q,u];
            if nout>=2
                t=[];
            end
        case 1
            S=cell2mat(S);
            l=length(S);
            auxi=cell(1,l);
            % vectorizer ?
            for i=1:l
                auxi{i}=0:1;
            end
            indd=logical(produit_cartesien(auxi));
            if nout>=2
                t=indd;
            else
                t=[];
            end
            lb=size(indd,1);
            P=cell(1,lb);
            for i=1:lb
                P{i}=S(indd(i,:));
            end
        case 2
            % quasi copier-coller du prcdent !
            S=cell2mat(S);
            l=length(S);
            P=cell(1,2^l);
            P{1}=[];
            k=1;
            for i=1:l
                indd=nchoosek(1:l,i);
                for j=1:size(indd)
                    k=k+1;
                    P{k}=S(indd(j,:));
                end
            end
            if nout>=2
                t=[];
            end
    end
end
if nout>=3
    m=length(P);
    if cho==1
        A=cell(m,2);
    else
        A=cell(m,1);
    end
    for i=1:m
        if cho==1
            auxi=[];
            for j=1:length(S)
                if t(i,j)
                    auxi=[auxi,'1'];
                else
                    auxi=[auxi,'0'];
                end
            end
            A{i,1}=auxi;
        end
        if cho==1
            if isempty(P{i})
                A{i,2}='\emptyset';
            else
                A{i,2}=affiche_ensemble03(P{i},1,[],[],0);
            end
        else
            if isempty(P{i})
                A{i,1}='\emptyset';
            else
                A{i,1}=affiche_ensemble03(P{i},1,[],[],0);
            end
        end
    end
end